@@ -20,7 +20,9 @@ from celery import current_app, group, signals, states
 from celery._state import _task_stack
 from celery.app.task import Context
 from celery.app.task import Task as BaseTask
-from celery.exceptions import Ignore, InvalidTaskError, Reject, Retry
+from celery.exceptions import (BackendGetMetaError, Ignore, InvalidTaskError,
+                               Reject, Retry)
+from celery.result import AsyncResult
 from celery.utils.log import get_logger
 from celery.utils.nodenames import gethostname
 from celery.utils.objects import mro_lookup
@@ -46,8 +48,15 @@ __all__ = (
     'setup_worker_optimizations', 'reset_worker_optimizations',
 )
 
+from celery.worker.state import successful_requests
+
 logger = get_logger(__name__)
 
+#: Format string used to log task receipt.
+LOG_RECEIVED = """\
+Task %(name)s[%(id)s] received\
+"""
+
 #: Format string used to log task success.
 LOG_SUCCESS = """\
 Task %(name)s[%(id)s] succeeded in %(runtime)ss: %(return_value)s\
@@ -159,9 +168,13 @@ class TraceInfo:
 
     def handle_error_state(self, task, req,
                            eager=False, call_errbacks=True):
-        store_errors = not eager
         if task.ignore_result:
             store_errors = task.store_errors_even_if_ignored
+        elif eager and task.store_eager_result:
+            store_errors = True
+        else:
+            store_errors = not eager
+
         return {
             RETRY: self.handle_retry,
             FAILURE: self.handle_failure,
@@ -316,7 +329,17 @@ def build_tracer(name, task, loader=None, hostname=None, store_errors=True,
     ignore_result = task.ignore_result
     track_started = task.track_started
     track_started = not eager and (task.track_started and not ignore_result)
-    publish_result = not eager and not ignore_result
+
+    # #6476
+    if eager and not ignore_result and task.store_eager_result:
+        publish_result = True
+    else:
+        publish_result = not eager and not ignore_result
+
+    deduplicate_successful_tasks = ((app.conf.task_acks_late or task.acks_late)
+                                    and app.conf.worker_deduplicate_successful_tasks
+                                    and app.backend.persistent)
+
     hostname = hostname or gethostname()
     inherit_parent_priority = app.conf.task_inherit_parent_priority
 
@@ -381,9 +404,31 @@ def build_tracer(name, task, loader=None, hostname=None, store_errors=True,
             except AttributeError:
                 raise InvalidTaskError(
                     'Task keyword arguments is not a mapping')
-            push_task(task)
+
             task_request = Context(request or {}, args=args,
                                    called_directly=False, kwargs=kwargs)
+
+            redelivered = (task_request.delivery_info
+                           and task_request.delivery_info.get('redelivered', False))
+            if deduplicate_successful_tasks and redelivered:
+                if task_request.id in successful_requests:
+                    return trace_ok_t(R, I, T, Rstr)
+                r = AsyncResult(task_request.id, app=app)
+
+                try:
+                    state = r.state
+                except BackendGetMetaError:
+                    pass
+                else:
+                    if state == SUCCESS:
+                        info(LOG_IGNORED, {
+                            'id': task_request.id,
+                            'name': get_task_name(task_request, name),
+                            'description': 'Task already completed successfully.'
+                        })
+                        return trace_ok_t(R, I, T, Rstr)
+
+            push_task(task)
             root_id = task_request.root_id or uuid
             task_priority = task_request.delivery_info.get('priority') if \
                 inherit_parent_priority else None
@@ -560,9 +605,9 @@ def _signal_internal_error(task, uuid, args, kwargs, request, exc):
         del tb
 
 
-def _trace_task_ret(name, uuid, request, body, content_type,
-                    content_encoding, loads=loads_message, app=None,
-                    **extra_request):
+def trace_task_ret(name, uuid, request, body, content_type,
+                   content_encoding, loads=loads_message, app=None,
+                   **extra_request):
     app = app or current_app._get_current_object()
     embed = None
     if content_type:
@@ -582,12 +627,9 @@ def _trace_task_ret(name, uuid, request, body, content_type,
     return (1, R, T) if I else (0, Rstr, T)
 
 
-trace_task_ret = _trace_task_ret  # noqa: E305
-
-
-def _fast_trace_task(task, uuid, request, body, content_type,
-                     content_encoding, loads=loads_message, _loc=None,
-                     hostname=None, **_):
+def fast_trace_task(task, uuid, request, body, content_type,
+                    content_encoding, loads=loads_message, _loc=None,
+                    hostname=None, **_):
     _loc = _localized if not _loc else _loc
     embed = None
     tasks, accept, hostname = _loc
@@ -622,8 +664,6 @@ def report_internal_error(task, exc):
 
 def setup_worker_optimizations(app, hostname=None):
     """Setup worker related optimizations."""
-    global trace_task_ret
-
     hostname = hostname or gethostname()
 
     # make sure custom Task.__call__ methods that calls super
@@ -649,16 +689,11 @@ def setup_worker_optimizations(app, hostname=None):
         hostname,
     ]
 
-    trace_task_ret = _fast_trace_task
-    from celery.worker import request as request_module
-    request_module.trace_task_ret = _fast_trace_task
-    request_module.__optimize__()
+    app.use_fast_trace_task = True
 
 
-def reset_worker_optimizations():
+def reset_worker_optimizations(app=current_app):
     """Reset previously configured optimizations."""
-    global trace_task_ret
-    trace_task_ret = _trace_task_ret
     try:
         delattr(BaseTask, '_stackprotected')
     except AttributeError:
@@ -667,8 +702,7 @@ def reset_worker_optimizations():
         BaseTask.__call__ = _patched.pop('BaseTask.__call__')
     except KeyError:
         pass
-    from celery.worker import request as request_module
-    request_module.trace_task_ret = _trace_task_ret
+    app.use_fast_trace_task = False
 
 
 def _install_stack_protection():
