@@ -278,19 +278,24 @@ class Backend:
                                  traceback=traceback, request=request)
 
     def chord_error_from_stack(self, callback, exc=None):
-        # need below import for test for some crazy reason
-        from celery import group  # pylint: disable
         app = self.app
         try:
             backend = app._tasks[callback.task].backend
         except KeyError:
             backend = self
+        # We have to make a fake request since either the callback failed or
+        # we're pretending it did since we don't have information about the
+        # chord part(s) which failed. This request is constructed as a best
+        # effort for new style errbacks and may be slightly misleading about
+        # what really went wrong, but at least we call them!
+        fake_request = Context({
+            "id": callback.options.get("task_id"),
+            "errbacks": callback.options.get("link_error", []),
+            "delivery_info": dict(),
+            **callback
+        })
         try:
-            group(
-                [app.signature(errback)
-                 for errback in callback.options.get('link_error') or []],
-                app=app,
-            ).apply_async((callback.id,))
+            self._call_task_errbacks(fake_request, exc, None)
         except Exception as eb_exc:  # pylint: disable=broad-except
             return backend.fail_from_current_stack(callback.id, exc=eb_exc)
         else:
