@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Celery Signals.
 
 This module defines the signals (Observer pattern) sent by
@@ -11,7 +10,6 @@ functions are called whenever a signal is called.
 
     :ref:`signals` for more information.
 """
-from __future__ import absolute_import, unicode_literals
 
 from .utils.dispatch import Signal
 
